#!/usr/bin/env python
#-*- coding: utf-8 -*-
#
# The MIT License (MIT)
# 
# Copyright (c) 2013 Lukas Märdian <maerdian@in.tum.de>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import csv, sys, math, os

gSUM = []
gSUMt = []
gSUMp = []

def evaluate(l):
    mean = sum(l)/len(l) if len(l)>0 else -1
    stddev = 0
    for e in l:
        stddev += math.sqrt((e-mean)*(e-mean))
    stddev = stddev/len(l) if len(l)>0 else -1 #stddev
    l.sort()
    if len(l) > 0:
        median = l[int(math.floor(len(l)/2.0))] #lower median
    else:
        median = None
    return (mean,stddev,median)

def run():
    global idmap
    global gSUM, gSUMt, gSUMp

    HZ = 6.0 # IR sample rate = 6Hz
    
    csvfile_ir = open(IR_FILE, 'rb')
    csvfile_us = open(US_FILE, 'rb')
    ir = csv.reader(csvfile_ir, delimiter=',')
    us = csv.reader(csvfile_us, delimiter=',')
    #skip headings
    ir.next() # dtheta,dphi,amplitude,frame,beacon1,beacon2
    us.next() # start,end,dtheta,dphi,amplitude,receiver,sender

    ir = [row for row in ir]
    first_ir_frame = int(ir[0][3])
    last_ir_frame  = int(ir[-1][3])
    frame_count = 0
    while int(ir[frame_count][3]) == first_ir_frame:
        frame_count += 1
    #print frame_count

    SUM = []  #amplitude
    SUMt = [] #dtheta
    SUMp = [] #dphi

    for row in us:
        start_frame = int(math.ceil(float(row[0])*HZ))
        end_frame = int(math.floor(float(row[1])*HZ))
        beacon1 = idmap[int(row[5])]
        beacon2 = idmap[int(row[6])]
        #print "frames:",start_frame,"to",end_frame,"ids:",beacon1,beacon2,

        if start_frame < first_ir_frame or end_frame > last_ir_frame:
            continue
        s = ((start_frame - first_ir_frame)*frame_count)
        e = ((end_frame - first_ir_frame)*frame_count)

        # aggregate the IR frame, which corresponds to the current US row in tmp
        tmp = []
        for i in xrange(s,e+1):
            if i<=last_ir_frame and int(ir[i][3]) in xrange(start_frame,end_frame+1) and int(ir[i][4]) == beacon1 and int(ir[i][5]) == beacon2:
                tmp.append(ir[i])
        #print len(tmp),
        #tmp = filter(lambda row: int(row[3]) in xrange(start_frame,end_frame+1) and \
        #        int(row[4])==beacon1 and int(row[5])==beacon2, ir)
        #print len(tmp)

        #calculate the IR std.dev (dtheta, dphi, amplitude)
        lst = [float(x[2]) for x in tmp] #amplitude
        lst_t = [float(x[0]) for x in tmp] #dtheta
        lst_p = [float(x[1]) for x in tmp] #dphi
        (amplitude, samplitude, mamplitude) = evaluate(lst)
        (dtheta, sdtheta, mdtheta) = evaluate(lst_t)
        (dphi, sdphi, mdphi) = evaluate(lst_p)
        #print "amplitude"
        #print (amplitude, samplitude, mamplitude)
        #print "dtheta"
        #print (dtheta, sdtheta, mdtheta)
        #print "dphi"
        #print (dphi, sdphi, mdphi)
        #print

        #consider only one IR measurement instead of the whole frame
        if len(tmp) > 0:
            tmpx = []
            #index = 0 #FRONT element
            index = int(math.floor(len(tmp)/2.0)) #MIDDLE element
            #index = len(tmp)-1 #TAIL element
            tmpx.append(tmp[index])
            tmp = tmpx
        else:
            continue

        lst = [float(x[2]) for x in tmp] #amplitude
        lst_t = [float(x[0]) for x in tmp] #dtheta
        lst_p = [float(x[1]) for x in tmp] #dphi

        #print int(round(lst[0])), row[4].strip()

        if len(lst) != 0:
            distance_ir = sum(lst)/len(lst)
            SUM.append( abs(abs(distance_ir/10) - abs(float(row[4])/10)) ) #local
            gSUM.append( abs(abs(distance_ir/10) - abs(float(row[4])/10)) ) #global
            theta_ir = sum(lst_t)/len(lst_t)
            if row[2].strip()!="ERROR":
                SUMt.append( abs(abs(theta_ir) - abs(float(row[2]))) ) #local
                gSUMt.append( abs(abs(theta_ir) - abs(float(row[2]))) ) #global
            phi_ir = sum(lst_p)/len(lst_p)
            if row[3].strip()!="ERROR":
                SUMp.append( abs(abs(phi_ir) - abs(float(row[3]))) ) #local
                gSUMp.append( abs(abs(phi_ir) - abs(float(row[3]))) ) #global
            #print round(distance/10),
            #print round(float(row[4])/10)
        #sys.stdout.write(".")
        #sys.stdout.flush()

#======================
# LOCAL evaluation
#======================

    (amplitude, samplitude, mamplitude) = evaluate(SUM)
    (dtheta, sdtheta, mdtheta) = evaluate(SUMt)
    (dphi, sdphi, mdphi) = evaluate(SUMp)

    #print "amplitude (cm):",amplitude, "std.dev:", samplitude, "median:", mamplitude
    #print "dtheta   (deg):",dtheta, "std.dev:", sdtheta, "median:", mdtheta
    #print "dphi     (deg):",dphi, "std.dev:", sdphi, "median:", mdphi

# Application main entry
if __name__ == '__main__':
    DPATH = '/media/Daten/Bachelorarbeit-Experiment/data/'
    #Date, time, participants, START, REAL_START, IR-US mapping
    INPUT = [
['2013-07-02', '1500', 4, 1372771533, 1372771605, '3:10:11:2'], #REAL_START fixed
['2013-07-02', '1600', 4, 1372774900, 1372774964, '11:2:10:3'],
['2013-07-03', '1500', 4, 1372857521, 1372857606, '4:11:10:2'],
['2013-07-03', '1600', 2, 1372861004, 1372861093, '10:11'],
['2013-07-03', '1730', 2, 1372866442, 1372866459, '10:11'],
['2013-07-04', '1100', 3, 1372929596, 1372929670, '2:11:10'],
['2013-07-04', '1200', 2, 1372932946, 1372932997, '11:10'],
['2013-07-11', '1500', 3, 1373548806, 1373548884, '9:11:10'], #REAL_START fixed
['2013-07-11', '1630', 4, 1373554140, 1373554267, '11:8:9:10'],
['2013-07-15', '1300', 4, 1373887107, 1373887203, '11:9:10:3'],
['2013-07-15', '1630', 3, 1373899678, 1373899755, '3:10:11'],
['2013-07-15', '1730', 3, 1373903465, 1373903471, '10:11:3'],
['2013-07-17', '1200', 3, 1374056316, 1374056405, '10:11:9'],
['2013-07-17', '1530', 3, 1374068891, 1374068956, '10:4:11']
#['2013-07-02', '1100', 3, 1372757322, 1372757444, '3:10:11'], #FAIL (sync at the end)
#['2013-07-11', '1200', 2, 1373537995, 1373538036, '11:10'], #FAIL
#['2013-07-17', '1300', 2, 1374059915, 1374059971, '11:10'], #FAIL
]

    #use xrange to check several rotation angles
    for angle in xrange(47,47+1):

        for e in INPUT:
            os.system('python normalizer.py '+str(e[3])+' '+str(e[4])+' '+str(e[2])+' '+DPATH+e[0]+'_'+e[1]+'/ultraschall/ '+str(angle)+' > us_test.csv 2>/dev/null')
            if os.path.exists(DPATH+e[0]+'_'+e[1]+'/auswertung/'+e[0]+'_'+e[1]+'_Lukas_splus-fixed.csv'):
                IR_FILE = DPATH+e[0]+'_'+e[1]+'/auswertung/'+e[0]+'_'+e[1]+'_Lukas_splus-fixed.csv'
            else:
                IR_FILE = DPATH+e[0]+'_'+e[1]+'/auswertung/'+e[0]+'_'+e[1]+'_Lukas_splus.csv'
            US_FILE = 'us_test.csv'
            IR_US_MAPPING = e[5]
            #print '======================================================================'
            #sys.stdout.write(e[0]+' '+e[1]+' ( ')
            # US sensor - IR marker mapping:
            #    index = US sensor ID
            #    value = IR marker ID
            idmap = [-1,-1,-1,-1]
            irmap = IR_US_MAPPING.split(':')
            #print 'Using the following US<->IR mapping:'
            for i in xrange(0,len(irmap)):
                idmap[i] = int(irmap[i])
                #sys.stdout.write('US'+str(i)+':IR'+str(idmap[i])+' ') 
            #print ")"

            run()

        #global evaluation
        print '======================================================================'
        print 'GLOBAL EVALUATION:'
        (amplitude, samplitude, mamplitude) = evaluate(gSUM)
        (dtheta, sdtheta, mdtheta) = evaluate(gSUMt)
        (dphi, sdphi, mdphi) = evaluate(gSUMp)

        print 'amplitude (cm),dtheta (deg),dphi (deg)'
        sys.stdout.write(str(amplitude)+','+str(math.degrees(dtheta))+','+str(math.degrees(dphi))+' (mean)\n')
        sys.stdout.write(str(samplitude)+','+str(math.degrees(sdtheta))+','+str(math.degrees(sdphi))+' (std.dev)\n')
        sys.stdout.write(str(mamplitude)+','+str(math.degrees(mdtheta))+','+str(math.degrees(mdphi))+' (median)\n')

        #print "rot,dphi mean, dphi stddev"
        #sys.stdout.write(str(angle)+','+str(math.degrees(dphi))+','+str(math.degrees(sdphi))+'\n')

        gSUM = []
        gSUMt = []
        gSUMp = []
