#!/usr/bin/env python
#-*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab
#
# The MIT License (MIT)
# 
# Copyright (c) 2013 Lukas Märdian <maerdian@in.tum.de>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import pygame, time, sys, thread

#---------- High Resolution POSIX Timing Functions ----------#
import ctypes, os
CLOCK_MONOTONIC = 1 # see <linux/time.h>
RELATIVE = 0
ABSOLUTE = 1

class timespec(ctypes.Structure):
    _fields_ = [
        ('tv_sec', ctypes.c_long),
        ('tv_nsec', ctypes.c_long)
    ]

librt = ctypes.CDLL('librt.so.1', use_errno=True)

clock_nanosleep = librt.clock_nanosleep
clock_nanosleep.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.POINTER(timespec), ctypes.POINTER(timespec)]
def nanosleep(sec,nsec,flags):
    request = timespec()
    request.tv_sec = sec
    request.tv_nsec = nsec
    remain = timespec()
    if clock_nanosleep(CLOCK_MONOTONIC, flags, ctypes.pointer(request), ctypes.pointer(remain)) != 0:
        errno_ = ctypes.get_errno()
        raise OSError(errno_, os.strerror(errno_))
    return remain.tv_sec + remain.tv_nsec * 1e-9

clock_gettime = librt.clock_gettime
clock_gettime.argtypes = [ctypes.c_int, ctypes.POINTER(timespec)]
def monotime():
    t = timespec()
    if clock_gettime(CLOCK_MONOTONIC, ctypes.pointer(t)) != 0:
        errno_ = ctypes.get_errno()
        raise OSError(errno_, os.strerror(errno_))
    return (t.tv_sec,t.tv_nsec)

def monotonic_time():
    """ Helper to return time as float instead of tuple """
    sec,nsec = monotime()
    return sec + nsec * 1e-9
#------------------------------------------------------------#

INTERVAL = 0.3
INTERVAL_nsec = int(INTERVAL*1e9)

def plus_one(sec,nsec):
    """ Helper to add INTERVAL to the (given) monotime """
    nsec += INTERVAL_nsec
    if nsec >= 1000000000:
      sec += 1
      nsec -= 1000000000
    return (sec,nsec)

def timestamp():
    print "press ENTER for a timestamp"
    while True:
        raw_input()
        t = time.time()
        l = time.localtime(t)
        print "%.6f -- %04d-%02d-%02d %02d:%02d:%02d" % (t, l[0], l[1], l[2], l[3], l[4], l[5])


thread.start_new_thread(timestamp, ())
pygame.mixer.pre_init(48000, -16, 1, 8)
pygame.init()

f0 = pygame.mixer.Sound('noise2.0.wav')
f1 = pygame.mixer.Sound('noise2.1.wav')
f2 = pygame.mixer.Sound('noise2.2.wav')

t = time.time()
l = time.localtime(t)
print "%.6f -- %04d-%02d-%02d %02d:%02d:%02d" % (t, l[0], l[1], l[2], l[3], l[4], l[5])

if __name__ == '__main__':
    try:
        NUM = int(sys.argv[1])
    except IndexError:
        NUM = None

    if len(sys.argv) == 1 or NUM==None:
        print "Usage: %s NUM_AGENTS"
        sys.exit(1)


sec,nsec = monotime()
while True:
    for i in xrange(0,6):
        f0.play()
        sec,nsec = plus_one(sec,nsec)
        nanosleep(sec,nsec,ABSOLUTE)
    #f1.play()
    #sec,nsec = plus_one(sec,nsec)
    #nanosleep(sec,nsec,ABSOLUTE)
    for i in xrange(0,NUM*6-6):
        f1.play()
        sec,nsec = plus_one(sec,nsec)
        nanosleep(sec,nsec,ABSOLUTE)

pygame.quit()

