#!/usr/bin/env python
#-*- coding: utf-8 -*-
#
# The MIT License (MIT)
# 
# Copyright (c) 2013 Lukas Märdian <maerdian@in.tum.de>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import csv, sys, math
from collections import Counter

#Global Vars
buf = [[],[],[],[]] #Buffer for up to 4 Agents
lst = [] #List containing all measured values
sectors = [-4,-3,-2,-1,0,1,2,3,4]
sectors_rev = [-x for x in sectors]

def sec_to_idx(sector):
    return sector+4

def map_sector(reverse, modes):
    if reverse:
        mapping = sectors_rev
    else:
        mapping = sectors

    if len(modes) == 1:
        if 1 in modes:
            return mapping[sec_to_idx(-4)]
        elif 5 in modes:
            return mapping[sec_to_idx(-2)]
        elif 2 in modes or 3 in modes:
            return mapping[sec_to_idx(0)]
        elif 4 in modes:
            return mapping[sec_to_idx(2)]
        elif 0 in modes:
            return mapping[sec_to_idx(4)]
        else:
            return -255 #ERR
    elif len(modes) == 2:
        if 1 in modes and 5 in modes:
            return mapping[sec_to_idx(-3)]
        elif 5 in modes and (3 in modes or 2 in modes):
            return mapping[sec_to_idx(-1)]
        elif 2 in modes and 3 in modes:
            return mapping[sec_to_idx(0)]
        elif 4 in modes and (3 in modes or 2 in modes):
            return mapping[sec_to_idx(1)]
        elif 0 in modes and 4 in modes:
            return mapping[sec_to_idx(3)]
        elif 0 in modes and (3 in modes or 2 in modes):
            return mapping[sec_to_idx(2)]
        elif 1 in modes and (3 in modes or 2 in modes):
            return mapping[sec_to_idx(-2)]
        else:
            return -255 #ERR
    elif len(modes) == 3:
        if 1 in modes and 5 in modes and (3 in modes or 2 in modes):
            return mapping[sec_to_idx(-2)]
        elif 0 in modes and 4 in modes and (3 in modes or 2 in modes):
            return mapping[sec_to_idx(2)]
        elif 0 in modes and 2 in modes and 3 in modes:
            return mapping[sec_to_idx(2)]
        elif 1 in modes and 2 in modes and 3 in modes:
            return mapping[sec_to_idx(-2)]
        elif 2 in modes and 3 in modes and 4 in modes:
            return mapping[sec_to_idx(1)]
        elif 2 in modes and 3 in modes and 5 in modes:
            return mapping[sec_to_idx(-1)]
        else:
            return -255 #ERR
    elif len(modes) == 4:
        if 0 in modes and 2 in modes and 3 in modes and 4 in modes:
            return mapping[sec_to_idx(2)]
        elif 1 in modes and 2 in modes and 3 in modes and 5 in modes:
            return mapping[sec_to_idx(-2)]
        else:
            return -255 #ERR
    else:
        return -255 #ERR

def sector_to_dphi(sector):
    global AGENT_ROTATION
    if sector != -255:
        dphi = [1, (7/8.0), (6/8.0), (5/8.0), (4/8.0), (3/8.0), (2/8.0), (1/8.0), 0]
        angle = (1+dphi[sector+4])*math.pi
        #angle = (dphi[sector+4])*math.pi
        angle = angle - AGENT_ROTATION #agent orientation VS sensorbox orientation
    else:
        angle = -255 #ERR
    return angle

def sectors_to_dtheta(self, remote):
    if self <= remote:
        sign = 1
    else:
        sign = -1
    return ((1-(abs((self+4)-(remote+4))/8.0))*math.pi)*sign

def procede_file(FILE, time_diff):
    global buf, lst, AGENTS
    i,j = (0,0)
    file_line = 1
    ID = FILE[-5:-4] #parse file name
    csvfile = open(FILE, 'rb')
    reader = csv.reader(csvfile, delimiter=';')
    reader.next() #skip heading (1/2)
    file_line+=1 #skip heading (2/2)
    x = reader.next()[0].split(' ') #first line with some meta information
    file_line+=1
    SELF_START = float(x[0])
    # if sensor was attached upside down, manually add REVERSED to the 2nd line of raw data
    # in this case we need a differen sensors/modes -> sector mapping
    if len(x) == 3 and x[2] == 'REVERSE':
        REVERSE = True
        sys.stderr.write('WARN: sensor reversed (ID: '+ID+') -- adjusting mappings...\n')
    else:
        REVERSE = False
    sub = SELF_START+time_diff #used to normalize all timestamps, to begin at 0.000
    for row in reader:
        row = [x.strip() for x in row]
        #if row[1][:1] != 's':
        if i != int(ID):
            tstamp   = float(row[0])-sub
            s_agent  = int(row[1][1:2])
            s_sens   = int(row[1][3:])
            # recognize and fix a problem, where i,j shifted due to a wrong error routine in a previous version of SSPtool
            if s_agent != i or s_sens != j:
                sys.stderr.write(
'WARN: agent/sensor shift in file \''+FILE+'\':\n'+
'    Line '+str(file_line)+': '+str(s_agent)+' '+str(s_sens)+' - '+'Fixed it: '+str(i)+' '+str(j)+'\n')
                s_agent = i
                s_sens = j
            r_agent  = int(ID)
            if 's' in row[3]:
                r_sens = -1 #ERROR: skipped value
            else:
                r_sens   = int(row[3])
            distance = int(row[2][:-3])
            buf[s_agent].append([tstamp,s_agent,s_sens,r_agent,r_sens,distance])
            #print tstamp
            #print 's:', s_agent, s_sens, 'r:', r_agent, r_sens, 'd:', distance
            #print row
            if s_sens == 5:
                #Timing
                start_time = buf[s_agent][0][0]
                end_time   = buf[s_agent][5][0]
                if start_time >= 0 and end_time > start_time:
                    #Recv sensors
                    r = [x[4] for x in buf[s_agent]]
                    r = filter(lambda e: e != -1, r) #Filter -1 (errors)
                    #Modalwerte
                    c = Counter(r)
                    if len(c.most_common(1)) == 0:
                        modes = [-1]
                    else:
                        maxi = c.most_common(1)[0][1]
                        modes = [x[0] for x in filter(lambda e: e[1]==maxi, c.most_common())]
                    #Distances
                    d = [int(x[5]) for x in buf[s_agent]]
                    #d = []
                    #for x in buf[s_agent]:
                    #    if x[4] in modes: #only values which arrived on mode sensors
                    #        d.append(x[5])
                    d = filter(lambda e: e != -1, d) #Filter -1 (errors)
                    #d = min(d)
                    if len(d) == 0:
                        m = -1
                    #elif len(d) == 1:
                    #    m = d[0]
                    elif len(d) <= 2:
                        #Mittelwert
                        m = sum(d)/len(d)
                    else:
                        #Mittelwert ohne Ausreißer
                        m = (sum(d)-max(d)-min(d))/(len(d)-2)
                    t = (start_time, end_time, r_agent, s_agent, m, map_sector(REVERSE,modes))
                    lst.append(t)
                    #print 'start: %8.3f, end: %8.3f, recv: %d, send: %d, dist: %3d, sectors: %s' % t
                #Clear buffer
                buf[s_agent] = []
        else:
            s_agent  = int(row[1][1:2])
            s_sens   = int(row[1][3:])
            # recognize and fix a problem, where i,j shifted due to a wrong error routine in a previous version of SSPtool
            if s_agent != i or s_sens != j:
                sys.stderr.write('WARN: agent/sensor shift in file \''+FILE+'\':\n'+
'    Line '+str(file_line)+': '+str(s_agent)+' '+str(s_sens)+' - '+'Fixed it: '+str(i)+' '+str(j)+'\n')

        i = (i+1) % AGENTS
        if i == 0:
            j = (j+1) % 6
        file_line+=1

# Application main entry
if __name__ == '__main__':
    try:
        HOST_START = float(sys.argv[1])
    except IndexError:
        HOST_START = None
    try:
        REAL_START = float(sys.argv[2])
    except IndexError:
        REAL_START = None
    try:
        AGENTS = int(sys.argv[3])
    except IndexError:
        AGENTS = None
    try:
        FOLDER = sys.argv[4]
    except IndexError:
        FOLDER = None
    try:
        AGENT_ROTATION = math.radians(int(sys.argv[5]))
    except:
        AGENT_ROTATION = math.radians(47) #47 degree yields best dphi

    if len(sys.argv) < 5:
        print "Usage: %s HOST_START REAL_START AGENTS FOLDER" % sys.argv[0]
        sys.exit(1)

    time_diff  = REAL_START-HOST_START
    for i in xrange(0,AGENTS):
        procede_file(FOLDER+'data_'+str(i)+'.csv', time_diff)

    lst = sorted(lst)
    #print 'start; end; recv; send; dist; sector'
    #for row in lst:
    #    print '%7.3f; %7.3f; %d; %d; %3d; %2d' % (row[0], row[1], row[2], row[3], row[4], row[5])

##### PHASE 2

    i = 0
    done_ids = [] # rows already processed
    res = [] # start, end, dtheta, dphi, amplitude, recv, send
    for row in lst:
        if not (i in done_ids):
            j = i+1
            recv = row[2]
            send = row[3]
            #search send-recv <-> recv-send partner
            while j<len(lst) and (recv != lst[j][3] or send != lst[j][2]):
                j += 1
            if j != len(lst):
                row2 = lst[j]
                sector = int(row[5])
                sector2 = int(row2[5])
                dphi  = sector_to_dphi(sector) if sector != -255 else "ERROR"
                dphi2 = sector_to_dphi(sector2) if sector2 != -255 else "ERROR"
                if dphi == "ERROR" or dphi2 == "ERROR":
                    dtheta = "ERROR"
                    dtheta2 = "ERROR"
                else:
                    dtheta = sectors_to_dtheta(sector, sector2)
                    dtheta2 = -dtheta
                #calculate the amplitude mean
                distance = (row[4]+row2[4])/2.0
                a = (row[0], row[1], dtheta, dphi, distance*10, row[2], row[3])
                b = (row2[0], row2[1], dtheta2, dphi2, distance*10, row2[2], row2[3])
                res.append(a)
                res.append(b)
                done_ids.append(i)
                done_ids.append(j)
                #print a
                #print b
        i += 1

    print 'start,end,dtheta,dphi,amplitude,receiver,sender'
    for row in res:
        if "ERROR" in row:
            if row[3] == "ERROR":
                print '%7.3f, %7.3f, %13s, %12s, %4d, %d, %d' % row
            else:
                print '%7.3f, %7.3f, %13s, %e, %4d, %d, %d' % row
        else:
            print '%7.3f, %7.3f, %13e, %e, %4d, %d, %d' % row
